# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import logging
import time

from cmf.monitor import schema
from cmf.monitor.abstract_hbase_monitor import AbstractHbaseMonitor
from cmf.monitor.base import DebugStatusReport
from cmf.monitor.regionserver.json_parser import JsonParser
from cmf.monitor.json_metrics_utils import ParseError, add_jvm_metrics
from cmf.monitor.proc_metrics_utils import add_proc_metrics
from cmf.monitor.role_update import RoleUpdate

LOG = logging.getLogger('MasterMonitor')

_ROLETYPE = 'REGIONSERVER'

# Section and key names in cloudera-monitor.properties
# Since 3.7
_SECTION = 'regionserver'
_SERVICE_NAME = 'service_name'
_ROLE_NAME = 'role_name'
_HOST = 'host'
_HBASE_REGIONSERVER_INFO_PORT = 'hbase_regionserver_info_port'
_COLLECT_INTERVAL = 'collect_interval'
# Added in CM 4.1
_LOG_DIR = 'log_dir'

class RegionServerMonitor(AbstractHbaseMonitor):

  def __init__(self, daemon, registry):
    AbstractHbaseMonitor.__init__(self,
                                  daemon,
                                  registry,
                                  _SECTION,
                                  _HBASE_REGIONSERVER_INFO_PORT)
    self._last_conf_path = None
    self._metrics_tuple = (None, None, None)
    self._last_reported_timestamp = None
    self._last_collection_ended = None
    self._last_collection_length = None
    self._role_dir_monitors = [ ]

    # Config data
    self._name = None
    self._service_name = None
    self._log_dir = None

  def __str__(self):
    return "RegionServerMonitor for %s" % (self._name,)

  def _reload_config(self, confparse):
    """Returns True/False for success"""
    try:
      # Extraction
      name = confparse.get(_SECTION, _ROLE_NAME)
      service = confparse.get(_SECTION, _SERVICE_NAME)
      log_dir = confparse.get_with_default(_SECTION, _LOG_DIR, None)
      host = confparse.get(_SECTION, _HOST)
      port = confparse.getint(_SECTION, _HBASE_REGIONSERVER_INFO_PORT)
    except ValueError, ex:
      self.record_error(ex)
      LOG.error("Value error in %s: %s" % (confparse.path, ex))
      return False

    metrics_url = self._compute_metrics_url(host, port)
    if metrics_url is None:
      return False

    # Assign
    self._name = name
    self._service_name = service
    self._log_dir = log_dir
    self._metrics_url = metrics_url

    # Make monitors for the log directory and the data directories.
    self._role_dir_monitors = [ ]
    if self._log_dir is not None:
      self.create_dir_monitor(self._log_dir, self._role_dir_monitors)

    return True

  def collect(self):
    """Overrides AbstractMonitor. Collect RegionServer metrics. Invoked 
       asynchronously."""
    start = time.time()

    # Reload config if necessary
    confparse = self.conf_parser
    if self._last_conf_path != confparse.path:
      if not self._reload_config(confparse):
        return
      self._last_conf_path = confparse.path

    # Collect directory info
    for mon in self._role_dir_monitors:
      mon.collect()

    # If our pid value is None, it means that the associated process is not
    # running. If this is the case, we skip metric collection.
    if self._pid is None:
      self._metrics_tuple = (None, None, None)
      return

    LOG.debug("Fetching RegionServer metrics at '%s'" % (self._metrics_url,))
    self._metrics_tuple = self.collect_metrics_from_url(self._metrics_url)

    self._last_collection_ended = time.time()
    self._last_collection_length = self._last_collection_ended - start

  def parse_metrics(self, json):
    return JsonParser(json)

  def get_role_update(self):
    """Returns a RoleUpdate to be sent to the firehose"""
    update = RoleUpdate(name=self._name,
                        roletype=_ROLETYPE,
                        service_name=self._service_name)

    if self.add_metrics_from_webserver(self._metrics_tuple,
                                       self._last_reported_timestamp,
                                       update):
      self._last_reported_timestamp = update.timestamp

    add_proc_metrics(update, self._pid, LOG)

    for mon in self._role_dir_monitors:
      update.add_dir_update(mon.get_role_dir_update())

    return update

  def add_webserver_metrics_to_role_update(self, update, metrics):
    # The metrics page is sometimes empty if the Master happens to
    # be flushing the metrics when we fetch it.
    if len(metrics.regionserver) == 0:
      return

    update.add_metric(schema.HBASE_FLUSH_QUEUE_SIZE,
                      metrics.regionserver.flushQueueSize)
    update.add_metric(schema.HBASE_COMPACTION_QUEUE_SIZE,
                      metrics.regionserver.compactionQueueSize)
    update.add_metric(schema.RS_BLOCK_CACHE_HIT_RATIO,
                      metrics.regionserver.blockCacheHitRatio)
    update.add_metric(schema.RS_STORE_FILE_INDEX_SIZE_MB,
                      metrics.regionserver.storefileIndexSizeMB)
    update.add_metric(schema.RS_STORES,
                      metrics.regionserver.stores)
    update.add_metric(schema.RS_STORE_FILES,
                      metrics.regionserver.storefiles)
    update.add_metric(schema.RS_MEMSTORE_SIZE_MB,
                      metrics.regionserver.memstoreSizeMB)
    update.add_metric(schema.RS_FS_READ_LATENCY,
                      metrics.regionserver.fsReadLatency_avg_time)
    update.add_metric(schema.RS_FS_SYNC_LATENCY,
                      metrics.regionserver.fsSyncLatency_avg_time)

    update.add_metric(
      schema.HBASE_OP_PUT_COUNT, metrics.rpc.put_num_ops)
    update.add_metric(
      schema.HBASE_OP_PUT_LATENCY, metrics.rpc.put_avg_time)
    update.add_metric(
      schema.HBASE_OP_MULTIPUT_COUNT, metrics.rpc.multiPut_num_ops)
    update.add_metric(
      schema.HBASE_OP_MULTIPUT_LATENCY, metrics.rpc.multiPut_avg_time)
    update.add_metric(
      schema.HBASE_OP_CHECKANDPUT_COUNT, metrics.rpc.checkAndPut_num_ops)
    update.add_metric(
      schema.HBASE_OP_CHECKANDPUT_LATENCY, metrics.rpc.checkAndPut_avg_time)
    update.add_metric(
      schema.HBASE_OP_MULTI_COUNT, metrics.rpc.multi_num_ops)
    update.add_metric(
      schema.HBASE_OP_MULTI_LATENCY, metrics.rpc.multi_avg_time)
    update.add_metric(
      schema.HBASE_OP_GET_COUNT, metrics.rpc.get_num_ops)
    update.add_metric(
      schema.HBASE_OP_GET_LATENCY, metrics.rpc.get_avg_time)
    update.add_metric(
      schema.HBASE_OP_ICV_COUNT, metrics.rpc.incrementColumnValue_num_ops)
    update.add_metric(
      schema.HBASE_OP_ICV_LATENCY, metrics.rpc.incrementColumnValue_avg_time)

    add_jvm_metrics(update, metrics)

  def get_debug_status_report(self):
    return DebugStatusReport(config_items=self.conf_parser.items(_SECTION),
                             errors=(self.last_error,),
                             last_collection_ended=self._last_collection_ended,
                             last_collection_length=self._last_collection_length)

  def get_subject_type(self):
    return "REGIONSERVER"
