// (c) Copyright 2011-2012 Cloudera, Inc. All rights reserved.
define([
	"cloudera/Util",
	"cloudera/common/DateUtil"
], function(Util, DateUtil) {

return Class.extend({

	/**
	 * Template method that the child class should implement.
	 */
	createConfig: function() {
		return {};
	},

	/**
	 * If the charts are initialized, return the charts's object model.
	 * otherwise, return the config object that we are constructing.
	 */
	getConfig: function() {
		if (this.chart && this.chart.objectModel) {
			// the initial config should be removed from this.
			// because the chart.objectModel should take over.
			this.config = null;
			return this.chart.objectModel;
		} else {
			return this.config;
		}
	},

	/**
	 * Template method to initialize the start/end time,
	 * which should arrive as part of options in the initialize method.
	 * Can be overwritten.
	 */
	initializeTime: function() {
	},

	/**
	 * Template method to initialize the metric data,
	 * which should arrive as part of options in the initialize method.
	 * Can be overwritten.
	 */
	initializeData: function() {
	},
	
	_preinitialize: function() {
	},

	/**
	 * Starting point.
	 */
	init: function(options, id){
		// console.log("AnyStockLineChartBase.initialize");
		try {
			this.options = options;

			this._preinitialize();
			
			// initialize time must happen first
			// because it is independent of the configuration.
			this.initializeTime();

			// initialize the JSON configuration object.
			this.config = this.createConfig();

			// initialize information for displaying the charts.
			this.initializeData();

			var chart = createAnyChartStock();
			chart.setConfig(this.config);
			chart.needConfig = true;

			this._addCallbacks(chart);
			this.chart = chart;
			chart.wMode = "opaque";
			chart.width = options.width || "100%";
			// this avoids a vertical scroll bar.
			chart.height = options.height || "99%";
			chart.write(id);

			// console.log("AnyStockLineChartBase.initialize done");
		} catch (ex) {
			console.log(ex);
		}

		return this;
	},

	_addCallbacks: function(chart){
		chart.onChartDataLoad = $.proxy( this.onChartDataLoad, this );
		chart.onChartDraw = $.proxy( this.onChartDraw, this );
		chart.onSelectedRangeChange = $.proxy( this.onSelectedRangeChange, this );
		chart.onAnnotationDrawingStart = $.proxy( this.onAnnotationDrawingStart, this );
		chart.onAnnotationDrawingFinish = $.proxy( this.onAnnotationDrawingFinish, this );
		chart.onAnnotationSelect = $.proxy( this.onAnnotationSelect, this );
		chart.onAnnotationDeselect = $.proxy( this.onAnnotationDeselect, this );
		chart.onChartMouseMove = $.proxy( this.onChartMouseMove, this );
		chart.onChartMouseDown = $.proxy( this.onChartMouseDown, this );
		chart.onChartMouseOut = $.proxy( this.onChartMouseOut, this );
		chart.onChartMouseUp = $.proxy( this.onChartMouseUp, this );
		chart.onEventMarkerSelect = $.proxy( this.onEventMarkerSelect, this );
		chart.onMergedEventMarkerSelect = $.proxy( this.onMergedEventMarkerSelect, this );
		chart.onEventMarkerMouseOver = $.proxy( this.onEventMarkerMouseOver, this );
		chart.onMergedEventMarkerMouseOver = $.proxy( this.onMergedEventMarkerMouseOver, this );
		chart.onEventMarkerMouseOut = $.proxy( this.onEventMarkerMouseOut, this );
		chart.onMergedEventMarkerMouseOut = $.proxy( this.onMergedEventMarkerMouseOut, this );
	},

	// All these methods are necessary as placeholders here.
	// They should be overwritten by subclasses.
	onChartDataLoad: function(){
	},

	onChartDraw: function(){
	},

	onSelectedRangeChange: function(startDate, endDate, phase, sender){
	},

	onAnnotationDrawingStart: function(id){
	},

	onAnnotationDrawingFinish: function(id){
	},

	onAnnotationSelect : function(id){
	},

	onAnnotationDeselect : function(id){
	},

	onChartMouseMove : function(chartId, date){
	},

	onChartMouseDown : function(chartId, date){
	},

	onChartMouseOut : function(chartId){
	},

	onChartMouseUp : function(chartId, date){
	},

	onEventMarkerSelect: function(eventMarker) {
	},
	
	onEventMarkerMouseOver: function(eventMarker) {
	},

	onEventMarkerMouseOut: function() {
	},

	onMergedEventMarkerSelect: function(eventMarkers) {
	},

	onMergedEventMarkerMouseOver: function() {
	},

	onMergedEventMarkerMouseOut: function() {
	},

	/**
	 * Returns the difference in milli-seconds,
	 * between two date objects.
	 */
	delta: function(d1, d2) {
		return DateUtil.delta(d1, d2);
	},

	min: function(d1, d2) {
		return DateUtil.min(d1, d2);
	},

	max: function(d1, d2) {
		return DateUtil.max(d1, d2);
	},

	avg: function(d1, d2) {
		return DateUtil.avg(d1, d2);
	},

	same: function(d1, d2) {
		return DateUtil.same(d1, d2);
	},

	/**
	 * Calculates the timezone offset in hours.
	 */
	getTimezoneOffset: function() {
		var MS_IN_ONE_HOUR = 3600000;
		return Util.getServerTimezoneOffset() / MS_IN_ONE_HOUR;
	},

	/**
	 * Zooms the chart to a start/end date.
	 * Note, this is in the browser's own timezone.
	 * We need to convert them to AnyChart's internal timezone, which is only GMT.
	 */
	zoomTo: function(startDate, endDate) {
		this.chart.zoomTo(this.toChartDate(startDate), this.toChartDate(endDate));
	},

	/**
	 * Converts a date into AnyChart's internal timezone.
	 * TODO: the timezone offset should be cached.
	 */
	toChartDate: function(date){
		var result = new Date(date.getTime() + this.getTimezoneOffset() * 60 * 60 * 1000);
		return result;
	},

	/**
	 * Converts a date from AnyChart's internal timezone.
	 * TODO: the timezone offset should be cached.
	 */
	fromChartDate: function(chartDate){
		var result = new Date(chartDate.getTime() - this.getTimezoneOffset() * 60 * 60 * 1000);
		return result;
	}
});

});
